<?PHP
/**
* Blog entry files structure
* <?PHP
* $entry = array();
* ?>
*
* Blog array structure
* array(
*	"name" => username,
*	"timestamp" => unix timestamp,
*	"title" => the blog title,
*	"content" => the blog entry (pagedata),
*	"categories" => array("category1", "category2"),
*	"personal" => boolean, defining wether the entry is personal or not,
*	"viewlevel" => integer, describing the viewing allowance level,
*	"commentlevel" => integer, describing the commenting allowance level,
*	"comments" => comment data for comment_handler
*	)
*/
class blogger_entry extends lockableClass
{
	var $username;
	var $timestamp;
	var $filename;
	var $dataFolder;
	
	var $title;
	var $content;
	var $categories;
	var $published;
	
	var $personal;
	var $viewLevel;
	var $commentLevel;
	var $comments;
	
	var $parentUser; ///< @var blogger_user Reference to the user owning this entry
	var $edit = false;
	
	/**
	 * Construct a new blogger_entry instance for the user in question
	 * If the timestamp is omitted, it means we are creating a new blog entry - just assign the current time
	 * 
	 * @param blogger_user   parentUser       Reference to the user who owns the entry
	 * @param int   timestamp        Optional. The timestamp for this entry. If none exists, we are creating a new entry.
	 */
	function blogger_entry( $parentUser, $timestamp = null )
	{
		global $userinfo_folder;
		$this->parentUser = &$parentUser;
		$this->username = $parentUser->username;
		$this->timestamp = $timestamp == null ? time() : $timestamp;
		$this->createFilename();
		
		// We should only attempt to load if we were given a timestamp to load...
		if( $timestamp != null )
			$this->load();
	}
	
	function createFilename()
	{
		global $userinfo_folder;
		$this->dataFolder = $userinfo_folder . "/" . $this->parentUser->username . "/blogger";
		$this->filename = $userinfo_folder . "/" . $this->parentUser->username . "/blogger/" . $this->timestamp . ".php";
		$this->lockfile = $this->filename . ".lock";
	}
	
	function load()
	{
		if( file_exists( $this->filename ) )
			include( $this->filename );
		
		$this->title = $entry["title"];
		$this->content = $entry["content"];
		$this->categories = $entry["categories"];
		$this->personal = $entry["personal"];
		$this->viewLevel = $entry["viewLevel"];
		$this->commentLevel = $entry["commentLevel"];
		$this->comments = $entry["comments"];
		$this->published = $entry["published"];
	}
	
	function save()
	{
		global $userinfo_folder;
		
		if( file_exists( $this->filename ) )
			unlink( $this->filename );
		
		$entry["title"] = $this->title;
		$entry["content"] = $this->content;
		$entry["categories"] = $this->categories;
		$entry["personal"] = $this->personal;
		$entry["viewLevel"] = $this->viewLevel;
		$entry["commentLevel"] = $this->commentLevel;
		$entry["comments"] = $this->comments;
		$entry["published"] = $this->published;
		
		if( !file_exists( $this->dataFolder ) )
			RecursiveMkdir( $this->dataFolder, 0775 );
		
		file_put_contents( $this->filename, array_export( $entry, "entry" ) );
	}
	
	function getGlobalID( $aswhat = "view" )
	{
		if( $aswhat == "view" )
			$aswhat = "";
		else
			$aswhat = "/" . $aswhat;
		
		return "user/" . $this->username . "/blogger" . $aswhat . "/entry/" . $this->timestamp;
	}
	
	function getFeedItem()
	{
		$title = $this->title;
		$link = globalIDToURL( $this->getGlobalID() );
		
		$splithere = strpos( $this->content, "---" );
		if( $splithere > 0 )
			$description = substr( $this->content, 0, $splithere );
		else
			$description = $this->content;
		
		$pubDate = $this->timestamp;
		
		return new FeedItem( $title, $link, $description, $pubDate, $link );
	}
	
	/**
	 * Check wether a user is allowed to view this blog entry
	 *
	 * @param    username         The username of the user who wishes to view the entry
	 *
	 * @return   A boolean describing wether or not the user is allowed
	 */
	function userCanView( $username )
	{
		if( $this->personal )
			return userAllows( $username, $this->parentUser->getParam( $allowsViewPersonal ) );
		else
			return userAllows( $username, $this->viewLevel );
	}
	
	function renderEditor()
	{
		global $globalID;
		$splitID = explode("/", $globalID);
		
		switch( $splitID[3] )
		{
		//  action: publish (toggle publish status)
		case "publish":
			if( $this->lock() )
			{
				$this->load();
				$this->published = !$this->published;
				$this->save();
				$this->unlock();
				header( "Location: " . globalIDtoURL( $this->parentUser->getGlobalID("edit") ) );
			}
			else
				$renderedContent = renderWarningBox( i18n("Could not lock"), i18n("The entry could not be locked! Please use your browser's back button and try again.") );
			break;
		//  action: delete
		case "delete":
			$dialog = new confirmDeleteDialog( $this->title, strtok( $this->content, "\n---\n" ), $this->getGlobalID("view") );
			
			if( $dialog->submitted )
			{
				if( $dialog->confirmed )
					unlink($this->filename);
				header( "Location: " . globalIDtoURL( $this->parentUser->getGlobalID("edit") ) );
			}
			
			$renderedContent = $dialog->render();
			break;
		//  action: none (edit entry)
		case "edit":
		default:
			$editor = new blogger_entryEditor( &$this );
			$renderedContent = "<form method=\"POST\" action=\"" . thisPageURL() . "\">" . $editor->render() . "</form>";
			break;
		}
		return $renderedContent;
	}
	
	/**
	 * Render the controls block for the blog entry
	 * 
	 * @return   The rendered controls
	 */
	function renderControls()
	{
		$view = drawCommand( i18n("View ##0##", array( "&raquo;" ) ), i18n("View the entry"), globalIDtoURL( $this->getGlobalID("view") ) );
		
		$commentHandler = new commentHandler( array(), 0, 0, 0 );
		
		$comments = i18n("Comments: ##0##", array( $commentHandler->countComments( $this->comments ) ) );
		
		if( $this->parentUser->parentEntries->edit )
		{
			$publishTitle = $this->published ? i18n("Unpublish") : i18n("Publish");
			$publishTip = $this->published ? i18n("Unpublish this entry") : i18n("Publish this entry");
			$publish = drawCommand( $publishTitle, $publishTip, globalIDtoURL( $this->getGlobalID("publish") ) );
			
			$edit = drawCommand( i18n("Edit ##0##", array( "&raquo;" ) ), i18n("Edit the entry"), globalIDtoURL( $this->getGlobalID("edit") ) );
			$delete = drawCommand( i18n("Delete ##0##", array( "&raquo;" ) ), i18n("Delete the entry"), globalIDtoURL( $this->getGlobalID("delete") ) );
		}
		
		$controls = "<div class=\"blogger-commands\">" . $comments . " " . $view . $publish . $edit . $delete . "</div>";
		
		return $controls;
	}
	
	function renderFull()
	{
		$date = "<div class=\"blogger-date\">" . formatTime( $this->timestamp ) . "</div>";
		$title = "<div class=\"blogger-title\">" . $this->title . "</div>";
		$content = "<div class=\"blogger-content\">" . parse_page_data( $this->content ) . "</div>";
		
		$commentHandler = new CommentHandler( $this->comments, $this->username, $this->commentLevel, 4 );
		
		$renderedContent = "<div class=\"blogger blogger-full\">" . $date . $title . $content . $commentHandler->renderComments() . $commentHandler->renderCommentsAdder() . "</div>";
		
		if( $commentHandler->commentsChanged )
		{
			if( $this->lock() )
			{
				$this->load();
				$this->comments = $commentHandler->comments;
				$this->save();
				$this->unlock();
			}
			
		}
		
		return $renderedContent;
	}
	
	function renderBlock()
	{
		$date = "<div class=\"blogger-date\">" . formatTime( $this->timestamp ) . "</div>";
		$title = "<div class=\"blogger-title\">" . $this->title . "</div>";
		
		$splithere = strpos( $this->content, "---" );
		if( $splithere > 0 )
			$content = substr( $this->content, 0, $splithere );
		else
			$content = $this->content;
		$content = "<div class=\"blogger-content\">" . parse_page_data( $content ) . "</div>";
		
		$controls = $this->renderControls();
		
		$renderedContent = "<div class=\"blogger blogger-block\">" . $date . $title . $content . $controls . "</div>";
		
		return $renderedContent;
	}
	
	function renderTiny()
	{
		$date = "<div class=\"blogger-date\">" . formatTime( $this->timestamp ) . "</div>";
		$title = "<div class=\"blogger-title\">" . $this->title . "</div>";
		$controls = $this->renderControls();
		
		$renderedContent = "<div class=\"blogger-tiny\">" . $date . $title . $controls . "</div>";
		return $renderedContent;
	}
	
	/**
	 *
	 *
	 * @param    block            Render the item in block mode (as opposed to the full entry)
	 * @param    tiny             Render the block in tiny mode (only title is shown)
	 *
	 * @return   String containing the rendered entry
	 */
	function render( $block = false, $tiny = false )
	{
		$renderedData = "";
		if( $this->edit )
			$renderedData = $this->renderEditor();
		else if( $tiny )
			$renderedData = $this->renderTiny();
		else if( $block )
			$renderedData = $this->renderBlock();
		else
			$renderedData = $this->renderFull();
		
		return $renderedData;
	}
}
?>