<?php
/**
 * Document Handling module for Travelsized CMS
 *
 * @Author	Dan Jensen <admin@leinir.dk>
 * @Version 0.1
 * @Date 30th Dec, 2006
 *
 * Data structure for documents folder:
 *
 * documents/
 *   content/
 *     data/*
 *     info/*
 *   categories/*
 *   documents.php <-- Contains all the documents' generic data, title, date, mdate, rating and blurb, for use with listings
 *   options.php
 *   orphans.php
 */

$modules_names[] = "Documents";
$modules_versions[] = "0.1";
$modules_descriptions[] = "A crosslinked story-tree style documents manager";

$modules_setup[] = "modulesetup_documents";
$profilemodules[] = "profilemodule_documents";

$modules[] = "parse_documents";

$page_help["&#92;document(txt1)"] = "Show a single document, with the id txt1";
$page_help["&#92;documentlink(txt1)"] = "Show a link to a document with the id txt1. The link text will be the document's title.";
$page_help["&#92;documentrating(txt1)"] = "Show a list of documents with the rating with the id txt1";
$page_help["&#92;documents()"] = "Show a list of documents. This is not normally used by users, in stead it is used internally by the documents module.";

$dox_infofolder = "$module_folder/documents/content/info";
if( !is_dir($dox_infofolder) )
	RecursiveMkdir( $dox_infofolder );
$dox_datafolder = "$module_folder/documents/content/data";
if( !is_dir($dox_datafolder) )
	RecursiveMkdir( $dox_datafolder );
$dox_categoryfolder = "$module_folder/documents/categories";
if( !is_dir($dox_categoryfolder) )
	RecursiveMkdir( $dox_categoryfolder );

// Set up some default options
$dox_options = array(
	"page_id" => 12, // If the user's not set anything up yet - show him the way back to module setup!
	"nextID" => 0, // The first document ID is zero, meaning that the first one actually returned by dox_nextID is 1
	"viewLevel" => 0, // Everybody can view
	"commentLevel" => 1, // Registered users can comment
	"editLevel" => 4, // Owner and admin only can edit
	"listingColumns" => array("title","mdate","authors"), // What columns to show in listings by default
	"rating" => 0, // Default rating, from the list below
	"ratings" => array(
		0 => "G rated",
		1 => "PG-13 rated",
		2 => "18 rated",
		3 => "X rated")
	);
$dox_optionfile = "$module_folder/documents/options.php";

if( file_exists( $dox_optionfile ) )
	include( $dox_optionfile );

if(!array_key_exists("listingColumns", $dox_options))
	$dox_options["listingColumns"] = array("title","date","authors"); // What columns to show in listings by default

$dox_listingcolumns = array(
	"title" => i18n("Title"),
	"date" => i18n("Creation date"),
	"mdate" => i18n("Last modified"),
	"rating" => i18n("Rating"),
	"owner" => i18n("Owner"),
	"authors" => i18n("Authors"));

//! A document which can be contained in no or more dox_category instances and link backwards and forwards to no or more dox_document instances
class dox_document extends lockableClass
{
	var $id;                           ///< The document's ID
	var $createdate;                   ///< When was the document created
	var $modifydate;                   ///< When was the document last edited (this includes comments)
	var $blurb;                        ///< A short description of the document, used in listings
	var $content;                      ///< Loaded on demand (that is, by render())
	var $comments;                     ///< commentsHandler data
	var $commentCount;                 ///< commentsHandler count of comments
	var $published;                    ///< Is the document supposed to be listed in document listings or not?
	
	var $backwardrel;                  ///< Array of IDs for previous parts to this document
	var $forwardrel;                   ///< Array of IDs for next parts to this document
	
	var $owner;                        ///< Who owns the document?
	var $viewLevel;                    ///< Viewing allowance level
	var $commentLevel;                 ///< Commenting allowance level
	var $editLevel;                    ///< Editing allowance level
	
	var $authors;                      ///< List, usernames or full names. If space exists, handled as username
	var $categories;                   ///< What categories are assigned to this document?
	var $rating;                       ///< What rating does this document have?
	
	///\private
	var $edit = false;                 ///< Wether or not we're editing
	var $infofile;                     ///< The filename of the data
	var $datafile;                     ///< The filename of the contents
	
	/**
	 * Constructor for the dox_document
	 *
	 * @param    id                  What is the document's ID? If not supplied, we're making a new document
	 * @param    document            Document data. If this exists, we will be saving this data to the new document
	 */
	function dox_document( $id = 0, $document = "" )
	{
		global $dox_infofolder, $dox_datafolder;
		
		$this->id = $id;
		
		$this->infofile = "$dox_infofolder/$id.php";
		$this->datafile = "$dox_datafolder/$id.php";
		$this->lockfile = "$dox_infofolder/$id.lock";
		if( file_exists( $this->infofile ) )
			include( $this->infofile );
		
		$this->title = $document["title"];
		$this->createdate = $document["createdate"];
		$this->modifydate = $document["modifydate"];
		$this->blurb = $document["blurb"];
		$this->published = $document["published"];
		$this->backwardrel = $document["backwardrel"];
		$this->forwardrel = $document["forwardrel"];
		$this->owner = $document["owner"];
		$this->viewLevel = $document["viewLevel"];
		$this->commentLevel = $document["commentLevel"];
		$this->comments = $document["comments"];
		$this->commentCount = $document["commentCount"];
		$this->editLevel = $document["editLevel"];
		$this->authors = $document["authors"];
		$this->categories = $document["categories"];
		$this->rating = $document["rating"];
		
		// If this is the case, we are creating a new document
		if( $id == 0 && is_array( $document ) )
		{
			// Get the ID for the new document
			$this->id = dox_nextID();
			// Remake the filenames with this new ID
			$this->infofile = "$dox_infofolder/" . $this->id . ".php";
			$this->datafile = "$dox_datafolder/" . $this->id . ".php";
			// Save this document - The notification is for the containing rating, someone may well have subscribed to that
			$this->save( i18n("The document was created") );
			// Add it to the all documents cache list
			$allDocs = new dox_alldocs();
			$allDocs->addDocument($this->id, $this->title, $this->owner, $this->authors, $this->createdate, $this->modifydate, $this->rating, $this->blurb, true);
		}
	}
	
	/**
	 * Load my content. This is potentially a lot of data, we don't want this in memory if we don't need it
	 */
	function loadContent()
	{
		if( file_exists( $this->datafile ) )
			include( $this->datafile );
	}
	
	/**
	 * Save the document's content. If we're passing content, then it was not set directly
	 *
	 * @param    content             Save this content to the document's content file. It also assigns this content to the object instance
	 */
	function saveContent( $content = "" )
	{
		if( $content != "" )
			$this->content = $content;
		
		if( file_exists( $this->datafile ) )
			unlink( $this->datafile );
		
		file_put_contents( $this->datafile, array_export( $this->content, "this->content" ) );
	}
	
	/**
	 * Save the document's information, with an optional message and comment, which is used to determine what to notify subscribers as
	 *
	 * @param    theMessage          What message to send to the subscribers, if it was not a comment
	 * @param    commenter           Who commented on the document?
	 */
	function save( $theMessage = "", $commenter = "" )
	{
		global $subscriptions;
		
		// Update info in global documents index and save
		$allDocs = new dox_alldocs();
		$allDocs->documents[$this->id]["title"] = $this->title;
		$allDocs->documents[$this->id]["blurb"] = $this->blurb;
		$allDocs->documents[$this->id]["owner"] = $this->owner;
		$allDocs->documents[$this->id]["authors"] = $this->authors;
		$allDocs->documents[$this->id]["date"] = $this->createdate;
		$allDocs->documents[$this->id]["mdate"] = $this->modifydate;
		$allDocs->documents[$this->id]["rating"] = $this->rating;
		$allDocs->documents[$this->id]["published"] = $this->published;
		$allDocs->documents[$this->id]["orphaned"] = ( count($this->categories) > 0 ) ? false : true;
		$allDocs->save();
		
		// Save self
		$document["title"] = $this->title;
		$document["createdate"] = $this->createdate;
		$document["modifydate"] = $this->modifydate;
		$document["blurb"] = $this->blurb;
		$document["published"] = $this->published;
		$document["owner"] = $this->owner;
		$document["viewLevel"] = $this->viewLevel;
		$document["commentLevel"] = $this->commentLevel;
		$document["commentCount"] = $this->commentCount;
		$document["editLevel"] = $this->editLevel;
		$document["rating"] = $this->rating;
		$document["comments"] = $this->comments;
		$document["backwardrel"] = $this->backwardrel;
		$document["forwardrel"] = $this->forwardrel;
		$document["authors"] = $this->authors;
		$document["categories"] = $this->categories;
		
		if( file_exists( $this->infofile ) )
			unlink( $this->infofile );
		
		if( $commenter != "" || $theMessage != "" )
		{
			if( $commenter != "" )
			{
				$subject = i18n("The document \"##0##\" was commented by ##0##", array( $this->title, getUserInfo($commenter, "name") ) );
				$message = i18n("A new comment was created by ##0## on the document \"##1##\". The comment was:##2####3##", array( getUserInfo($commenter, "name"), $this->title, "\n$theMessage", "\n---" . globalIDtoURL("documents/view/" . $this->id) ) );
			}
			else if ( $theMessage != "" )
			{
				$subject = i18n("The details for \"##0##\" was edited by ##0##", array( $this->title, getUserInfo(currentUser(), "name") ) );
				$message = i18n("The deils for the change by ##0## on the document \"##1##\" are as follows:##2####3##", array( getUserInfo($commenter, "name"), $this->title, "\n$theMessage", "\n---" . globalIDtoURL("documents/view/" . $this->id) ) );
			}
			// Notify categories...
			// Notify rating...
			$subscriptions->handle( "documents/rating/" . $this->rating, $this->owner, $this->viewLevel, $subject, $message );
			// Notify document
			$subscriptions->handle( "documents/view/" . $this->id, $this->owner, $this->viewLevel, $subject, $message );
		}
		
		file_put_contents( $this->infofile, array_export( $document, "document" ) );
	}
	
	/**
	 * Delete the document's information and data files
	 *
	 * @return   True if delete process was successful, false if not
	 */
	function delete()
	{
		if( userAllows( $this->owner, $this->editLevel ) )
		{
			// Delete own datafiles
			if( file_exists( $this->infofile ) )
				unlink( $this->infofile );
			if( file_exists( $this->datafile ) )
				unlink( $this->datafile );
			
			// Subscriptions need removing
			global $subscriptions;
			$subscriptions->removeGlobalID( "documents/view/" . $this->id );
			
			// Remove from alldocs cache
			$allDocs = new dox_alldocs();
			unset( $allDocs->documents[$this->id] );
			$allDocs->save();
			
			return true;
		}
		else
			return false;
	}
	
	/**
	 * NOT IMPLEMENTED
	 */
	function addToCategory( $category )
	{
		// Check if doc is in orphaned documents listing
			// Remove doc from orphaned documents listing if so
			// Save orphaned documents listing
		// Add document to category
		// Save category
		// Add category to document
		// Save document
	}
	
	/**
	 * NOT IMPLEMENTED
	 */
	function removeFromCategory( $category )
	{
		// Remove doc from category
		// Save category
		// Remove category from document
		// Save document
		// Check wether document is categorised
			// If not, add to orphaned documents listing
			// Save orphaned documents listing
	}
	
	/**
	 * Check wether the user currently logged in is allowed to edit the document
	 *
	 * @return   A boolean describing wether the current user can edit this document
	 */
	function userCanEdit()
	{
		if( userIsAllowed(currentUser(), "global_admin") )
			return true;
		
		if( userAllows( $this->owner, $this->editLevel ) )
			return true;
		
		if( in_array(currentUser(), $this->authors) )
			return true;
		
		return false;
	}
	
	/**
	 * Render this document's editor
	 *
	 * @return   A string containing the rendered document editor
	 */
	function renderEditor()
	{
		global $globalID, $page_title, $dox_options, $permissionlevels_array;
		$this->loadContent();
		$allDocs = new dox_alldocs();
		
		$splitID = explode( "/", $globalID );
		
		switch( $splitID[3] )
		{
		case "deleteprev":
		case "deletenext":
			$otherDoc = new dox_document( $splitID[4] );
			if( $_REQUEST["confirmDelete"] == "true" )
			{
				if( $this->lock() )
				{
					$this->load();
					$otherDoc->load();
					
					if( $splitID[3] == "deleteprev" )
					{
						unset( $this->backwardrel[array_search($splitID[4], $this->backwardrel)] );
						unset( $otherDoc->forwardrel[array_search($this->id, $this->forwardrel)] );
					}
					else
					{
						unset( $this->forwardrel[array_search($splitID[4], $this->forwardrel)] );
						unset( $otherDoc->backwardrel[array_search($this->id, $this->backwardrel)] );
					}
					
					$this->save( i18n("The document is no longer associated with \"##0##\" - ##1##", array($otherDoc->title, globalIDtoURL("documents/view/" . $this->id ) ) ) );
					$otherDoc->save( i18n("The document is no longer associated with \"##0##\" - ##1##", array($this->title, globalIDtoURL("documents/view/" . $otherDoc->id ) ) ) );
					
					$this->unlock();
					$otherDoc->unlock();
				}
				
				header( "Location: " . globalIDtoURL( "documents/edit/" . $this->id ) );
			}
			
			$title = ($splitID[3] == "deleteprev") ? i18n("Remove previous document") : i18n("Remove next document");
			$renderedContent = renderInformationBox($title,
				i18n("Are you sure you wish to remove the link between the documents ##0## and ##1##", array("&quot;" . $this->title . "&quot;", "&quot;" . $otherDoc->title . "&quot;")) . "<div class=\"wikicenteralign\">" .
				drawCommand( i18n("Confirm removal"), i18n("Confirm that you wish to remove the link between the two documents"), globalIDtoURL("documents/edit/" . $this->id . "/" . $splitID[3] . "/" . $splitID[4], array("confirmDelete" => "true")) ) . " " .
				drawCommand( i18n("Abort removal"), i18n("Abort the removal process"), globalIDtoURL("documents/edit/" . $this->id) ) . "</div>"
				, false);
			break;
		case "addprev":
		case "addnext":
			$panel = new optionsPanel("");
			$panel->saveTitle = i18n("Add document");
			$panel->showHeader = false;
			
			$values["document"] = $_POST["document"] ? $_POST["document"] : "";
			
			$availableDocs = array();
			foreach( $allDocs->documents as $key => $value )
			{
				if(
					( $splitID[3] == "addprev" && $value["id"] != $this->id && !in_array($value["id"], $this->backwardrel) ) ||
					( $splitID[3] == "addnext" && $value["id"] != $this->id && !in_array($value["id"], $this->forwardrel) )
					)
					$availableDocs[$value["id"]] = $value["title"];
			}
			
			$panel->addOption(
				i18n("Document to add"),
				i18n("Select the document you wish to add to the list of documents"),
				$values["document"],
				"document",
				"select",
				$availableDocs);
			
			if( $_POST["saveOptions"] )
			{
				$otherDoc = new dox_document( $values["document"] );
				
				if( $this->lock() )
				{
					$this->load();
					$otherDoc->load();
					
					if( $splitID[3] == "addprev" )
					{
						$this->backwardrel[] = $values["document"];
						$otherDoc->forwardrel[] = $this->id;
					}
					else
					{
						$this->forwardrel[] = $values["document"];
						$otherDoc->backwardrel[] = $this->id;
					}
					
					$this->save( i18n("The document was associated with \"##0##\" - ##1##", array($otherDoc->title, globalIDtoURL("documents/view/" . $this->id ) ) ) );
					$otherDoc->save( i18n("The document was associated with \"##0##\" - ##1##", array($this->title, globalIDtoURL("documents/view/" . $otherDoc->id ) ) ) );
					
					$this->unlock();
					$otherDoc->unlock();
				}
				
				header( "Location: " . globalIDtoURL( "documents/edit/" . $this->id ) );
			}
			
			$title = ($splitID[3] == "addprev") ? i18n("Add previous document") : i18n("Add next document");
			$renderedContent = renderInformationBox($title, $panel->render(), false);
			break;
		case "removeauthor":
			if( strpos( $this->authors[$splitID[4]], "\n" ) === false )
				$authorName = parse_profilelinks("%%" . $this->authors[$splitID[4]] . "%%");
			else
				$authorName = strtok($this->authors[$splitID[4]], "\n" );
			
			if( $_REQUEST["confirmDelete"] == "true" )
			{
				if( $this->lock() )
				{
					$this->load();
					unset( $this->authors[$splitID[4]] );
					$this->save( i18n("The author ##0## was removed from the document", array($authorName) ) );
					$this->unlock();
				}
				
				header( "Location: " . globalIDtoURL( "documents/edit/" . $this->id ) );
			}
			
			$title = ($splitID[3] == "deleteprev") ? i18n("Remove previous document") : i18n("Remove next document");
			
			$renderedContent = renderInformationBox(i18n("Remove author"),
				i18n("Are you sure you wish to remove the author ##0## from the document ##1##", array($authorName, "&quot;" . $this->title . "&quot;")) . "<div class=\"wikicenteralign\">" .
				drawCommand( i18n("Confirm removal"), i18n("Confirm that you wish to remove the author from the document"), globalIDtoURL("documents/edit/" . $this->id . "/removeauthor/" . $splitID[4], array("confirmDelete" => "true")) ) . " " .
				drawCommand( i18n("Abort removal"), i18n("Abort the removal process"), globalIDtoURL("documents/edit/" . $this->id) ) . "</div>"
				, false);
			break;
			break;
		case "addauthor":
			$panel = new optionsPanel("");
			$panel->saveTitle = i18n("Add author");
			$panel->showHeader = false;
			
			$values["user"] = $_POST["user"] ? $_POST["user"] : "";
			$values["name"] = $_POST["name"] ? $_POST["name"] : "";
			$values["url"] = $_POST["url"] ? $_POST["url"] : "";
			
			$availableDocs = array();
			$allUsers = allUsersArray();
			foreach( $allUsers as $key => $value )
				if( !in_array($value, $this->authors) )
					$availableUsers[$value] = getUserInfo( $value, "name" );
			
			if( count( $availableUsers ) > 0 )
			{
				$panel->addHeader( i18n("Either select a user"), "user" );
				$panel->addOption(
					i18n("Author to add"),
					i18n("Select the author you wish to add to the list of authors"),
					$values["user"],
					"user",
					"select",
					$availableUsers);
				$panel->addHeader( i18n("Or type in the author's details"), "external" );
			}
			$panel->addOption(
				i18n("Name"),
				i18n("The author's name"),
				$values["name"],
				"name",
				"text");
			$panel->addOption(
				i18n("URL"),
				i18n("Type in a valid URL for this author, for example http://www.somesite.com/ or mailto:user@email.com"),
				$values["url"],
				"url",
				"text");
			
			if( $_POST["saveOptions"] )
			{
				if( $values["name"] != "" )
					$author = $values["name"] . "\n" . $values["url"];
				else
					$author = $values["user"];
				
				if( $this->lock() )
				{
					$this->load();
					$this->authors[] = $author;
					$this->save( i18n("The author ##0## was added to the document", array($author) ) );
					$this->unlock();
				}
				
				header( "Location: " . globalIDtoURL( "documents/edit/" . $this->id ) );
			}
			
			$title = i18n("Add author");
			$renderedContent = renderInformationBox($title, $panel->render(), false);
			break;
		default:
			$panel = new optionsPanel("");
			$panel->saveTitle = i18n("Save document");
			$panel->showHeader = false;
			
			$values["title"] = $_POST["title"] == "" ? $this->title : stripslashes($_POST["title"]);
			$values["blurb"] = $_POST["blurb"] == "" ? $this->blurb : stripslashes($_POST["blurb"]);
			$values["rating"] = $_POST["rating"] == "" ? $this->rating : $_POST["rating"];
			$values["viewLevel"] = $_POST["viewLevel"] == "" ? $this->viewLevel : $_POST["viewLevel"];
			$values["commentLevel"] = $_POST["commentLevel"] == "" ? $this->commentLevel : $_POST["commentLevel"];
			$values["editLevel"] = $_POST["editLevel"] == "" ? $this->editLevel : $_POST["editLevel"];
			$values["content"] = $_POST["content"] == "" ? $this->content : stripslashes($_POST["content"]);
			$values["owner"] = $_POST["owner"] == "" ? $this->owner : $_POST["owner"];
			
			$panel->addOption(
				i18n("Title"),
				i18n("Title of the new document"),
				$values["title"],
				"title",
				"text");
			$panel->addOption(
				i18n("Blurb"),
				i18n("A short description of the document"),
				$values["blurb"],
				"blurb",
				"text");
			$panel->addOption(
				i18n("Rating"),
				i18n("What rating does this document fall under?"),
				$values["rating"],
				"rating",
				"select",
				$dox_options["ratings"]);
			
			if( userIsAllowed(currentUser(), "global_admin") )
				$panel->addOption(
					i18n("Document owner"),
					i18n("Who owns this document?"),
					$values["owner"],
					"owner",
					"select",
					allUsersArray(true, true));
			
			$panel->addOption(
				i18n("Viewing allowance level"),
				i18n("Who except ##0## is allowed to view this document?", array(getUserInfo($values["owner"], "name"))),
				$values["viewLevel"],
				"viewLevel",
				"select",
				$permissionlevels_array);
			$panel->addOption(
				i18n("Commenting allowance level"),
				i18n("Who except ##0## is allowed to comment on this document?", array(getUserInfo($values["owner"], "name"))),
				$values["commentLevel"],
				"commentLevel",
				"select",
				$permissionlevels_array);
			$panel->addOption(
				i18n("Editing allowance level"),
				i18n("Who except ##0## is allowed to edit this document?", array(getUserInfo($values["owner"], "name"))),
				$values["editLevel"],
				"editLevel",
				"select",
				$permissionlevels_array);
			$panel->addOption(
				i18n("Document contents"),
				"",
				$values["content"],
				"content",
				"pagedata");
			
			if( $_POST["saveOptions"] )
			{
				if( $this->lock() )
				{
					$this->load();
					
					$this->title = $values["title"];
					$this->modifydate = time();
					$this->blurb = $values["blurb"];
					$this->viewLevel = $values["viewLevel"];
					$this->commentLevel = $values["commentLevel"];
					$this->editLevel = $values["editLevel"];
					$this->rating = $values["rating"];
					
					if( userIsAllowed(currentUser(), "global_admin") )
						$this->owner = $values["owner"];
					
					$theReason = i18n("The main document details were changed.");
					if( $this->content != $values["content"] )
						$theReason .= " " . i18n("The contents of the document itself were also changed.");
					$this->content = $values["content"];
					
					$this->save( $theReason );
					$this->saveContent();
					
					$this->unlock();
				}
				
				header("Location: " . globalIDtoURL( "documents/view/" . $this->id ) );
			}
			
			// Title
			$page_title = $this->title;
			// Navigator - previous and next links side-by-side, with stats between them (rating, letter count, word count, lix, categories in list (with links))
			$prevDocs = "";
			foreach( $this->backwardrel as $key => $value )
				$prevDocs .= "
				<a href=\"" . globalIDtoURL("documents/edit/" . $this->id . "/deleteprev/$value") . "\">" . i18n("Remove ##0##", array($allDocs->documents[$value]["title"])) . "</a><br />";
			if( $prevDocs == "" )
				$prevDocs = i18n("No previous documents") . "<br />";
			$prevDocs .= drawCommand(i18n("Add previous document"), i18n("Add a document to the list of documents to read before this one"), globalIDtoURL("documents/edit/" . $this->id . "/addprev"));
				
			$nextDocs = "";
			foreach( $this->forwardrel as $key => $value )
				$nextDocs .= "
				<a href=\"" . globalIDtoURL("documents/edit/" . $this->id . "/deletenext/$value") . "\">" . i18n("Remove ##0##", array($allDocs->documents[$value]["title"])) . "</a><br />";
			if( $nextDocs == "" )
				$nextDocs = i18n("No next documents") . "<br />";
			$nextDocs .= drawCommand(i18n("Add next document"), i18n("Add a document to the list of documents to read next"), globalIDtoURL("documents/edit/" . $this->id . "/addnext"));
			
			$firstAuthor = true;
			foreach( $this->authors as $key => $value )
			{
				if( $firstAuthor == false )
					$renderedAuthors .= ", ";
				else
					$firstAuthor = false;
				
				if( strpos($value, "\n") === false )
					$renderedAuthors .= "<a href=\"" . globalIDtoURL("documents/edit/" . $this->id . "/removeauthor/" . $key) . "\" >" . getUserInfo($value, "name") . "</a>";
				else
				{
					list( $authorName, $authorURL ) = explode( "\n", $value);
					$renderedAuthors .= "<a href=\"" . globalIDtoURL("documents/edit/" . $this->id . "/removeauthor/" . $key) . "\" >" . $authorName . "</a>";
				}
			}
			if( $firstAuthor == false )
				$renderedAuthors .= ", ";
			$renderedAuthors .= drawCommand( i18n("Add author"), i18n("Add an author to the list of authors"), globalIDtoURL("documents/edit/" . $this->id . "/addauthor"));
			
			$wordcount = 1;
			$lettercount = 1;
			$lix = $lettercount / $wordcount;
			$categorylist = i18n("(not implemented)");
			
			$docStats = "
			<div class=\"documents-viewdoc-navigator-stats\">" . i18n("Authors: ##0##", array($renderedAuthors)) . "</div>
			<div class=\"documents-viewdoc-navigator-stats\">" . i18n("Categories: ##0##", array($categorylist)) . "</div>";
			
			$navigator = "
			<table class=\"documents-viewdoc-navigator\">
			<tr class=\"documents-viewdoc-navigator\">
				<td class=\"documents-viewdoc-navigator-previous\">$prevDocs</td>
				<td class=\"documents-viewdoc-navigator-stats\">$docStats</td>
				<td class=\"documents-viewdoc-navigator-next\">$nextDocs</td>
			</tr>
			</table>";
			$renderedContent .= $navigator;
			// Content
			$renderedContent .= "$editLink<div class=\"documents-viewdoc-content\">" . $panel->render() . "</div>";
			// Navigator again (same as above)
			$renderedContent .= $navigator;
		}
		return "<form action=\"" . thisPageURL() . "\" method=\"POST\">" . $renderedContent . "</form>";
	}
	
	/**
	 * Render the document as a table row
	 *
	 * @param    userColumns         The current user's preference for what columns to show ($key => $showcolumn)
	 *
	 * @return   A string containing the rendered table row block for this document
	 */
	function renderBlock($userColumns)
	{
		global $dox_options;
		$renderedData = "";
		if( $this->published || $this->edit )
		{
			$renderedData = "
			<tr class=\"documents-table\">";
			if( in_array( "title", $userColumns ) )
				$renderedData .= "
				<td class=\"documents-table\">" . $this->title . " (" . $this->commentCount . ")<small class=\"comment\">" . $this->blurb . "</small></td>";
			if( in_array( "date", $userColumns ) )
				$renderedData .= "
				<td class=\"documents-table\">" . formatTime($this->createdate) . "</td>";
			if( in_array( "mdate", $userColumns ) )
				$renderedData .= "
				<td class=\"documents-table\">" . formatTime($this->modifydate) . "</td>";
			if( in_array( "rating", $userColumns ) )
				$renderedData .= "
				<td class=\"documents-table\">" . $dox_options["ratings"][$this->rating] . "</td>";
			if( in_array( "owner", $userColumns ) )
				$renderedData .= "
				<td class=\"documents-table\">" . parse_profilelinks("%%" . $this->owner . "%%", "documents") . "</td>";
			// The view command is ALWAYS shown
			if( in_array( "authors", $userColumns ) )
			{
				$renderedData .= "
				<td class=\"documents-table\">";
				$firstAuthor = "";
				foreach( $this->authors as $key => $value )
				{
					$renderedData .= $firstAuthor;
					$renderedData .= parse_profilelinks("%%" . $value . "%%", "documents");
					$firstAuthor = ", ";
				}
				$renderedData .= "</td>";
			}
			$renderedData .= "
				<td class=\"documents-table-right\">" . drawCommand(i18n("View ##0##", array("&raquo;")), i18n("View ##0##", array($this->title)), globalIDtoURL("documents/view/" . $this->id) ) . "</td>
			</tr>";
		}
		return $renderedData;
	}
	
	/**
	 * Render the document
	 *
	 * @return   A string containing the rendered document
	 */
	function render()
	{
		global $page_title, $dox_options, $subscriptions;
		$trail = new breadcrumbs( i18n("Documents"), globalIDtoURL("documents") );
		$trail->addCrumb( $dox_options["ratings"][$this->rating], globalIDtoURL("documents/rating/" . $this->rating) );
		$trail->addCrumb( $this->title, globalIDtoURL("documents/view/" . $this->id) );
		if( $this->userCanEdit() && $this->edit )
			$renderedContent = $this->renderEditor();
		else
		{
			$this->loadContent();
			$allDocs = new dox_alldocs();
			// Title
			$page_title = $this->title;
			// Navigator - previous and next links side-by-side, with stats between them (rating, letter count, word count, lix, categories in list (with links))
			$prevDocs = "";
			foreach( $this->backwardrel as $key => $value )
				$prevDocs .= "
				<a href=\"" . globalIDtoURL("documents/view/" . $value) . "\" class=\"documents-viewdoc-navigator-previous\">" . $allDocs->documents[$value]["title"] . "</a><br />";
			if( $prevDocs == "" )
				$prevDocs = i18n("No previous documents") . "<br />";
				
			$nextDocs = "";
			foreach( $this->forwardrel as $key => $value )
				$nextDocs .= "
				<a href=\"" . globalIDtoURL("documents/view/" . $value) . "\" class=\"documents-viewdoc-navigator-next\">" . $allDocs->documents[$value]["title"] . "</a><br />";
			if( $nextDocs == "" )
				$nextDocs = i18n("No next documents") . "<br />";
			
			$wordcount = count_words($this->content);
			$lettercount = strlen($this->content);
			$categorylist = i18n("(not implemented)");
			
			$firstAuthor = true;
			foreach( $this->authors as $key => $value )
			{
				if( $firstAuthor == false )
					$renderedAuthors .= ", ";
				else
					$firstAuthor = false;
				
				if( strpos($value, "\n") === false )
					$renderedAuthors .= parse_profilelinks( "%%$value%%", "documents" );
				else
				{
					list( $authorName, $authorURL ) = explode( "\n", $value);
					$renderedAuthors .= "<a href=\"" . $authorURL . "\" >" . $authorName . "</a>";
				}
			}
			
			$commenthandler = new commentHandler($this->comments, $this->owner, $this->commentLevel, $this->editLevel);
			$commenthandler->thisPageURL = globalIDtoURL( "documents/view/" . $this->id );
			if( $commenthandler->commentsChanged )
			{
				$this->comments = $commenthandler->comments;
				$this->commentCount = $commenthandler->numberOfComments();
				$this->save( $commenthandler->lastComment, $commenthandler->lastCommenter );
			}
			
			$docStats = "
			<div class=\"documents-viewdoc-navigator-stats\">" . i18n("Rating: ##0##", array($dox_options["ratings"][$this->rating])) . "</div>
			<div class=\"documents-viewdoc-navigator-stats\">" . i18n("Authors: ##0##", array($renderedAuthors)) . "</div>
			<div class=\"documents-viewdoc-navigator-stats\">" . i18n("Document contains ##0## words and ##1## letters", array($wordcount, $lettercount)) . "</div>
			<div class=\"documents-viewdoc-navigator-stats\">" . i18n("Categories: ##0##", array($categorylist)) . "</div>";
			
			$navigator = "
			<table class=\"documents-viewdoc-navigator\">
			<tr class=\"documents-viewdoc-navigator\">
				<td class=\"documents-viewdoc-navigator-previous\">$prevDocs</td>
				<td class=\"documents-viewdoc-navigator-stats\">$docStats</td>
				<td class=\"documents-viewdoc-navigator-next\">$nextDocs</td>
			</tr>
			</table>";
			$renderedContent .= $navigator;
			// Content
			if( userAllows( $this->owner, $this->editLevel ) )
				$editLink = "<div class=\"documents-viewdoc-editlink\">" . drawCommand(i18n("Edit"), i18n("Edit the document"), globalIDtoURL("documents/edit/" . $this->id)) . "</div>";
			$renderedContent .= $editLink . $subscriptions->renderSubscribeControl( "documents/view/" . $this->id, false ) . "
				<div class=\"documents-viewdoc-content\">" . parse_page_data($this->content) . "</div>
				" . $commenthandler->renderComments() . $commenthandler->renderCommentsAdder();
			// Navigator again (same as above)
			$renderedContent .= $navigator;
		}
		return "<div class=\"documents-breadcrumbs\">" . $trail->render() . "</div>"  . $renderedContent;
	}
}

//! A list of documents
class dox_category
{
	var $id;                           ///< The category's ID
	var $parentToplevelCategory;       ///< The parent topcategory's ID
	
	var $title;                        ///< The category's title
	var $ids;                          ///< Array of dox_document IDs
	
	///\privatesec
	var $datafile;                     ///< The data file
	var $documents;                    ///< All the contained documents, $id => new dox_document()
	
	/**
	 * Constructor for the dox_category class
	 *
	 * @param    id                  What is this category's ID
	 * @param    parentID            What is the parent toplevel category's ID
	 */
	function dox_category( $id, $parentToplevelCategory )
	{
		global $dox_categoryfolder;
		$this->id = $id;
		$this->parentID = &$parentToplevelCategory;
		$this->datafile = "$dox_categoryfolder/" . $this->parentToplevelCategory . ".$id.php";
		// Load the actual data
		if( file_exists( $this->datafile ) )
			include( $this->datafile );
		
		$this->title = $category["title"];
		$this->ids = $category["ids"];
	}
	
	/**
	 * Save this category
	 */
	function save()
	{
		if( file_exists( $this->datafile ) )
			unlink( $this->datafile );
		
		$category["title"] = $this->title;
		$category["ids"] = $this->ids;
		
		file_put_contents( $this->datafile, array_export( $category, "category" ) );
	}
	
	/**
	 * Render the category
	 */
	function render()
	{
		// Cycle through all IDs and render them in block mode - fetch from global dox_alldocs
	}
}

//! A list of dox_category instances
class dox_topcategory
{
	var $id;                           ///< The top category's ID
	
	var $title;                        ///< The top category's title
	var $ids;                          ///< The contained categories' IDs
	
	var $categories;                   ///< The contained categories
	
	///\private
	var $datafile;                     ///< The data file
	
	/**
	 * Constructor for the dox_topcategory class
	 *
	 * @param    id                  The top level category's ID
	 */
	function dox_topcategory( $id )
	{
		global $dox_categoryfolder;
		
		$this->datafile = "$dox_categoryfolder/$id.php";
		
		if( file_exists($this->datafile) )
			include( $this->datafile );
		
		$this->title = $category["title"];
		$this->ids = $category["ids"];
		
		// Load all contained categories
		foreach($this->ids as $key => $value)
			$this->categories[$value] = new dox_category( $value, $this->id );
	}
	
	/**
	 * Saves the top level category
	 */
	function save()
	{
		if( file_exists( $this->datafile ) )
			unlink( $this->datafile );
		
		$category["title"] = $this->title;
		$category["ids"] = $this->ids;
		
		file_put_contents( $this->datafile, array_export( $category, "category" ) );
	}
	
	/**
	 * Deletes a category from the top level category
	 *
	 * @param    id                  The ID of the category you want to remove
	 */
	function deleteCategory( $id )
	{
		// Delete category file from disc
		if( file_exists( $this->categories[$id]->datafile ) )
			unlink( $this->categories[$id]->datafile );
		// Delete category from array
		unset( $this->categories[$id] );
		unset( $this->ids[array_search($id, $this->ids)] );
		// Save self
		$this->save();
	}
	
	/**
	 * Create a new category in this top level category
	 *
	 * @param    title               The title of the new category
	 */
	function newCategory( $title )
	{
		$theEnd = endKey( $this->ids );
		$this->ids[] = $theEnd + 1;
		$this->categories[$theEnd + 1] = new dox_category( $theEnd + 1, &$this);
		$this->categories[$theEnd + 1]->save();
		$this->save();
	}
	
	/**
	 * Renders the top level category
	 */
	function render()
	{
		// Cycle through categories and show the title with a link and the count of IDs contained
		foreach( $this->categories as $key => $category )
			$renderedContent .= "<p class=\"documents-listing-category\"><a class=\"documents-listing-category\" href=\"" . globalIDtoURL( $category->globalID ) . "\">" . $category->title . " (" . count($category->ids) . ")</p>";
		
		if( $renderedContent != "" )
			$renderedContent = "<p class=\"documents-listing-topcategory\">" . $this->title . "</p>" . $renderedContent;
		
		return $renderedContent;
	}
}

class dox_allcategories
{
	var $topcategories;                ///< Array containing instances of all top level categories
	
	function dox_allcategories()
	{
		global $dox_categoryfolder;
		$this->topcategories = array();
		// Load all top level galleries from disc
		$dir_handle = opendir( $dox_categoryfolder );
		while( $file = readdir( $dir_handle ) )
		{
			if( substr($file, 0, 1) != "."  )
			{
				$id = substr( $file, 0, -4 );
				// If there's a . in here, it means it's a category, not a top level category
				if( strpos( $id, "." ) === false )
					$this->topcategories[] = new dox_topcategory( $id );
			}
		}
		closedir($dir_handle);
	}
	
	function render()
	{
		global $dox_options;
		
		$renderedOptions = dox_renderListOptions();
		
		// render all the top categories
		foreach( $this->topcategories as $key => $topcategory )
			$renderedContent .= $topcategory->render();
		
		if( $renderedContent == "" )
		{
			$showThis = new dox_alldocs();
			$renderedContent = $showThis->render();
		}
		else
			$renderedContent .= $renderedOptions;
		
		return $renderedContent;
	}
}

//! An index of all the dox_document elements on the system
class dox_alldocs
{
	var $documents;                    ///< Array containing all the documents' generic data: title, owner, date, mdate, rating, blurb, published status, and wether they are orphaned - for use with listings
	var $sortby;                       ///< What to sort by. Accepted values: title, date, mdate, rating
	var $thisRatingOnly = null;        ///< If this is set, only this rating is shown
	
	///\private
	var $datafile;                     ///< The data file
	
	/**
	 * Constructor for the dox_alldocs class
	 */
	function dox_alldocs()
	{
		global $module_folder;
		
		$this->datafile = "$module_folder/documents/documents.php";
		$this->documents = array();
		if( file_exists( $this->datafile ) )
			include( $this->datafile );
		
		$this->sortby = getUserInfo( currentUser(), "dox_sortby", "title" );
		if( $_REQUEST["sortby"] != "" )
			$this->sortby = $_REQUEST["sortby"];
	}
	
	/**
	 * Adds a document to the global documents index cache
	 *
	 * @param    id                  The document's ID
	 * @param    title               The document's title (sortable)
	 * @param    owner               Who owns the document (sortable)
	 * @param    date                The creation date of this document (sortable)
	 * @param    modifydate          The last date on which this document was modified (sortable)
	 * @param    rating              The document's rating (sortable)
	 * @param    blurb               The document's short description text
	 * @param    orphan              Wether the document is listed in any categories or not (true if not listed)
	 */
	function addDocument( $id, $title, $owner, $authors, $date, $modifydate, $rating, $blurb, $orphan )
	{
		$this->documents[$id] = array( "id" => $id, "title" => $title, "owner" => $owner, "authors" => $authors, "date" => $date, "mdate" => $modifydate, "rating" => $rating, "blurb" => $blurb, "orphan" => $orphan );
		$this->save();
	}
	
	/**
	 * Removes a specific document from the global documents index cache
	 *
	 * @param    id                  The ID of the document to remove
	 */
	function removeDocument( $id )
	{
		unset( $this->documents[$id] );
		$this->save();
	}
	
	/**
	 * Save the global documents index cache
	 */
	function save()
	{
		if( file_exists( $this->datafile ) )
			unlink( $this->datafile );
		
		file_put_contents( $this->datafile, array_export( $this->documents, "this->documents" ) );
	}
	
	/**
	 * Render the listing of all documents
	 *
	 * @return   The rendered documents listing
	 */
	function render()
	{
		global $dox_options, $dox_listingcolumns, $globalID, $subscriptions;
		
		$trail = new breadcrumbs( i18n("Documents"), globalIDtoURL("documents") );
		
		
		// List all documents sorted by any of the things in the list, defaulting to title
		$splitID = explode("/", $globalID);
		$thisRatingOnly = $this->thisRatingOnly;
		if( $splitID[1] == "rating" && array_key_exists( 2, $splitID ) )
			$thisRatingOnly = $splitID[2];
		
		if( $thisRatingOnly != null )
		{
			$subscriptionControl = $subscriptions->renderSubscribeControl( "documents/rating/" . $thisRatingOnly, false );
			$trail->addCrumb( i18n($dox_options["ratings"][$thisRatingOnly]), globalIDtoURL("documents/rating/" . $thisRatingOnly));
			$shownItems = array();
			foreach( $this->documents as $key => $value )
			{
				if( $value["rating"] == $thisRatingOnly )
					$shownItems[] = $value;
			}
		}
		else
		{
			$subscriptionControl = $subscriptions->renderSubscribeControl( "documents", false );
			// Do the options thing - only if we're not listing a specific rating
			$renderedOptions = dox_renderListOptions();
			
			// Get the documents to be shown (show only those the user requsts by way of filtering out those with unwanted ratings
			$userShow = explode( ",", getUserInfo( currentUser(), "dox_showratings", implode( ",", array_keys($dox_options["ratings"] ) ) ) );
			$shownItems = array();
			foreach( $this->documents as $key => $value )
			{
				if( in_array($value["rating"], $userShow) )
					$shownItems[] = $value;
			}
		}
		
		// Figure out what columns we're showing...
		$userColumns = explode( ",", getUserInfo( currentUser(), "dox_showcolumns", implode( ",", array_values($dox_options["listingColumns"] ) ) ) );
		// ...and render those columns' headers
		$tableHeader = "
		<tr class=\"documents-table-header\">";
		foreach( $userColumns as $key => $value )
		{
			// Make sure we're going to sort in the correct order when you click on the header
			$thisSort = $this->sortby == $value ? $value . "rev" : $value;
			$tableHeader .= "<th class=\"documents-table documents-table-header-$value\"><a class=\"document-table-header\" href=\"" . thisPageURL(false, true, array("sortby" => "$thisSort")) . "\">" . $dox_listingcolumns[$value] . "</a></th>";
		}
		// Remember we need this column - otherwise the view command link will lack space
		$tableHeader .= "
			<th class=\"documents-table documents-table-header-right\">&nbsp;</th>
		</tr>";
		
		// Sort data by requested sort order
		usort( $shownItems, "dox_compare_" . $this->sortby );
		
		// Render each document
		$renderedItems = "";
		foreach( $shownItems as $key => $value )
		{
			$thisDoc = new dox_document( $value["id"] );
			$renderedItems .= $thisDoc->renderBlock($userColumns);
		}
		
		// Tell the user if we haven't got anything to show
		if( $renderedItems == "" )
			$renderedContent = renderInformationBox(i18n("No documents to show"), i18n("There are no documents fitting your request. There can be a number of causes for this. Either no documents exist in the system, or you have chosen a set of ratings all of which contain no documents. Please check your options below and try again."));
		else
			$renderedContent = "
		<table class=\"documents-table\">$tableHeader$renderedItems
		</table>";
		
		return "<div class=\"documents-breadcrumbs\">" . $trail->render() . "</div>" . $renderedContent . $renderedOptions;
	}
}

function dox_compare_title($x, $y) { return arrayColumnCompare($x, $y, "title"); }
function dox_compare_titlerev($x, $y) { return arrayColumnCompare($x, $y, "title", true); }
function dox_compare_date($x, $y) { return arrayColumnCompare($x, $y, "date"); }
function dox_compare_daterev($x, $y) { return arrayColumnCompare($x, $y, "date", true); }
function dox_compare_mdate($x, $y) { return arrayColumnCompare($x, $y, "mdate"); }
function dox_compare_mdaterev($x, $y) { return arrayColumnCompare($x, $y, "mdate", true); }
function dox_compare_owner($x, $y) { return arrayColumnCompare($x, $y, "owner"); }
function dox_compare_ownerrev($x, $y) { return arrayColumnCompare($x, $y, "owner", true); }
function dox_compare_authors($x, $y) { return arrayColumnCompare($x, $y, "authors"); }
function dox_compare_authorsrev($x, $y) { return arrayColumnCompare($x, $y, "authors", true); }
function dox_compare_rating($x, $y) { return arrayColumnCompare($x, $y, "rating"); }
function dox_compare_ratingrev($x, $y) { return arrayColumnCompare($x, $y, "rating", true); }

function parse_documents( $page_data )
{
	global $globalID, $page_id, $dox_options, $meta_headers, $module_folder;
	
	$loopcontrol = 0;
	$moduleCommandName = "document";
	while( $parameters = spotModuleCommand( $page_data, $moduleCommandName, &$loopcontrol ) )
	{
		if( $parameters == " " )
			$parameters = "";
		
		$showThis = new dox_document( $parameters );
		
		$replaceData = $showThis->render();
		$rendered = true;
		
		$page_data = str_replace( "\\$moduleCommandName($parameters)", $replaceData, $page_data );
	}
	
	$loopcontrol = 0;
	$moduleCommandName = "documentrating";
	while( ( $parameters = spotModuleCommand( $page_data, $moduleCommandName, &$loopcontrol ) ) !== false )
	{
		if( $parameters == " " )
			$parameters = "";
		
		$showThis = new dox_alldocs();
		$showThis->thisRatingOnly = $parameters == "" ? null : $parameters;
		
		$replaceData = $showThis->render();
		$rendered = true;
		
		$page_data = str_replace( "\\$moduleCommandName($parameters)", $replaceData, $page_data );
	}
	
	$loopcontrol = 0;
	$moduleCommandName = "documentlink";
	while( $parameters = spotModuleCommand( $page_data, $moduleCommandName, &$loopcontrol ) )
	{
		if( $parameters == " " )
			$parameters = "";
		
		$showThis = new dox_document( $parameters );
		
		$replaceData = "<a href=\"" . globalIDtoURL("documents/view/" . $showThis->id) . "\">" . $showThis->title . "</a>";
		$rendered = true;
		
		$page_data = str_replace( "\\$moduleCommandName($parameters)", $replaceData, $page_data );
	}
	
	$loopcontrol = 0;
	$moduleCommandName = "documents";
	while( $parameters = spotModuleCommand( $page_data, $moduleCommandName, &$loopcontrol ) )
	{
		if( $parameters == " " )
			$parameters = "";
		
		if( strlen($parameters) > 0 )
			$splitID = explode("/", $parameters);
		else
			$splitID = explode("/", $globalID);
		
		$replaceData = ""; // Let's initialize to an empty string...
		if( $page_id == $dox_options["page_id"] )
		{
			switch( $splitID[1] )
			{
				case "view":
				case "edit":
					if( array_key_exists( 2, $splitID ) )
					{
						// Show the document if we've got an ID
						$showThis = new dox_document( $splitID[2] );
						if( $splitID[1] == "edit" && $showThis->userCanEdit() )
							$showThis->edit = true;
					}
					else
					{
						// List all top level categories if we've not got an ID to show
						$showThis = new dox_allcategories();
					}
					break;
				case "category":
					if( array_key_exists( 2, $splitID ) )
					{
						// Get the top level gallery
						$theTopcategory = new dox_topcategory( $splitID[2] );
						if( array_key_exists( 3, $splitID ) )
						{
							// List documents in this category if we've got a category...
							$showThis = new dox_category( $splitID[3], $theTopcategory->id );
						}
						else
						{
							// Otherwise just show the top level category 
							$showThis = &$theTopcategory;
						}
					}
					else
					{
						// List all top level categories if we've not got an ID to show
						$showThis = new dox_allcategories();
					}
					break;
				case "rating":
					if( array_key_exists( 3, $splitID ) )
					{
						// List all documents with this rating
					}
					else
					{
						// List all top level categories if we've not got an ID to show
						$showThis = new dox_allcategories();
					}
					break;
				case "listall":
					// List all documents
					$showThis = new dox_alldocs();
					break;
				case "toplevel":
				default:
					// List all toplevel categories
					$showThis = new dox_allcategories();
			}
			$replaceData = $showThis->render();
			$rendered = true;
		}
		
		$page_data = str_replace( "\\$moduleCommandName($parameters)", $replaceData, $page_data );
	}
	
	if( $rendered )
		$meta_headers .= "
	<link rel=\"StyleSheet\" href=\"" . siteURL(true) . "$module_folder/documents/documents.css\" type=\"text/css\" />";
	
	return $page_data;
}

function profilemodule_documents()
{
	global $tabwidget, $meta_headers, $module_folder, $viewUser, $globalID, $permissionlevels_array, $dox_options;
	$splitID = explode( "/", $globalID );
	$meta_headers .= "
	<link rel=\"StyleSheet\" href=\"" . siteURL(true) . "$module_folder/documents/documents.css\" type=\"text/css\" />";
	
	if( currentUser() == $viewUser || isAllowed("global_user_edit") )
	{
		$tabwidget->addCommand( i18n("New"), i18n("Create a new document"), globalIDtoURL("user/$viewUser/documents/new") );
		$tabwidget->addCommand( i18n("Options"), i18n("Change the document options"), globalIDtoURL("user/$viewUser/documents/options") );
		$tabwidget->addCommand( i18n("View"), i18n("View ##0##'s documents", array(getUserInfo($viewUser, "name"))), globalIDtoURL("user/$viewUser/documents") );
	}
	
	switch( $splitID[3] )
	{
		// User's personal options, defaults for new documents - user/username/documents/options
		case "options":
			if( !userAllows( $theDocument->owner, $theDocument->editLevel ) )
				header("Location: " . globalIDtoURL("user/$viewUser/documents") );
			
			$panel = new optionsPanel("");
			
			$values["viewLevel"] = $_POST["viewLevel"] == "" ? getUserInfo($viewUser, "dox_viewLevel", $dox_options["viewLevel"]) : $_POST["viewLevel"];
			$values["commentLevel"] = $_POST["commentLevel"] == "" ? getUserInfo($viewUser, "dox_commentLevel", $dox_options["commentLevel"]) : $_POST["commentLevel"];
			$values["editLevel"] = $_POST["editLevel"] == "" ? getUserInfo($viewUser, "dox_editLevel", $dox_options["editLevel"]) : $_POST["editLevel"];
			
			$panel->addOption(
				i18n("Viewing allowance level"),
				i18n("Who except ##0## is allowed to view documents owned by ##0##?", array($viewUser)),
				$values["viewLevel"],
				"viewLevel",
				"select",
				$permissionlevels_array);
			$panel->addOption(
				i18n("Commenting allowance level"),
				i18n("Who except ##0## is allowed to view documents owned by ##0##?", array($viewUser)),
				$values["commentLevel"],
				"commentLevel",
				"select",
				$permissionlevels_array);
			$panel->addOption(
				i18n("Editing allowance level"),
				i18n("Who except ##0## is allowed to view documents owned by ##0##?", array($viewUser)),
				$values["editLevel"],
				"editLevel",
				"select",
				$permissionlevels_array);
			
			if( $_POST["saveOptions"] )
			{
				saveUserInfo($viewUser, "dox_viewLevel", $values["viewLevel"]);
				saveUserInfo($viewUser, "dox_commentLevel", $values["commentLevel"]);
				saveUserInfo($viewUser, "dox_editLevel", $values["editLevel"]);
			}
			
			$renderedContent = $panel->render();
			break;
		// Let user delete their documents - user/username/documents/delete/id
		case "delete":
			if( array_key_exists( 4, $splitID ) )
			{
				$theDocument = new dox_document( $splitID[4] );
				if( !userAllows( $theDocument->owner, $theDocument->editLevel ) )
					header("Location: " . globalIDtoURL("user/$viewUser/documents") );
				else
				{
					if( $_REQUEST["confirmDelete"] == "true" )
					{
						$theDocument->delete();
						header("Location: " . globalIDtoURL("user/$viewUser/documents") );
					}
					else
						$renderedContent = renderInformationBox(
							i18n("Delete document?"),
							i18n("Are you sure you wish to delete the document ##0##?", array("&quot;" . $theDocument->title . "&quot;")) .
								"<div class=\"wikicenteralign\">" .
								drawCommand(i18n("Confirm delete"), i18n("Comfirm deletion of ##0##", array($theDocument->title)), globalIDtoURL("user/$viewUser/documents/delete/" . $theDocument->id, array("confirmDelete" => "true"))) . " " .
								drawCommand(i18n("Abort delete"), i18n("Abort the deletion process"), globalIDtoURL("user/$viewUser/documents")) . "</div>",
							false);
				}
			}
			else
				header("Location: " . globalIDtoURL("user/$viewUser/documents") );
			break;
		// Let user publish and unpublish their documents - user/username/documents/togglepublish/id
		case "togglepublish":
			if( array_key_exists( 4, $splitID ) )
			{
				$theDocument = new dox_document( $splitID[4] );
				if( userAllows( $theDocument->owner, $theDocument->editLevel ) )
				{
					$theDocument->published = !$theDocument->published;
					$theDocument->save();
				}
			}
			header("Location: " . globalIDtoURL("user/$viewUser/documents") );
			
			break;
		// Let user edit their documents - user/username/documents/edit/id
		case "edit":
			if( array_key_exists( 4, $splitID ) )
			{
				if( !userAllows( $theDocument->owner, $theDocument->editLevel ) )
					header("Location: " . globalIDtoURL("user/$viewUser/documents") );
				else
				{
					$theDocument = new dox_document( $splitID[4] );
					$theDocument->edit = true;
					$renderedContent = $theDocument->renderEditor();
				}
			}
			else
				header("Location: " . globalIDtoURL("user/$viewUser/documents") );
			break;
		// Let user create new documents - user/username/documents/new
		case "new":
			if( !userAllows( $theDocument->owner, $theDocument->editLevel ) )
				header("Location: " . globalIDtoURL("user/$viewUser/documents") );
			
			$panel = new optionsPanel("");
			$panel->saveTitle = i18n("Create document");
			$panel->revertTitle = i18n("Reset to defaults");
			$panel->showHeader = false;
			
			$values["title"] = $_POST["title"] == "" ? i18n("New title") : stripslashes($_POST["title"]);
			$values["blurb"] = $_POST["blurb"] == "" ? i18n("A description of the document!") : stripslashes($_POST["blurb"]);
			$values["rating"] = $_POST["rating"] == "" ? $dox_options["rating"] : $_POST["rating"];
			$values["viewLevel"] = $_POST["viewLevel"] == "" ? getUserInfo($viewUser, "dox_viewLevel", $dox_options["viewLevel"]) : $_POST["viewLevel"];
			$values["commentLevel"] = $_POST["commentLevel"] == "" ? getUserInfo($viewUser, "dox_commentLevel", $dox_options["commentLevel"]) : $_POST["commentLevel"];
			$values["editLevel"] = $_POST["editLevel"] == "" ? getUserInfo($viewUser, "dox_editLevel", $dox_options["editLevel"]) : $_POST["editLevel"];
			$values["content"] = $_POST["content"] == "" ? i18n("Write the contents of your document in this field. Formatting help can be found below.") : stripslashes($_POST["content"]);
			
			$panel->addHeader(
				i18n("Create a new document"),
				"first");
			$panel->addOption(
				i18n("Title"),
				i18n("Title of the new document"),
				$values["title"],
				"title",
				"text");
			$panel->addOption(
				i18n("Blurb"),
				i18n("A short description of the document"),
				$values["blurb"],
				"blurb",
				"text");
			$panel->addOption(
				i18n("Rating"),
				i18n("What rating does this document fall under?"),
				$values["rating"],
				"rating",
				"select",
				$dox_options["ratings"]);
			$panel->addOption(
				i18n("Viewing allowance level"),
				i18n("Who except ##0## is allowed to view documents owned by ##0##?", array($viewUser)),
				$values["viewLevel"],
				"viewLevel",
				"select",
				$permissionlevels_array);
			$panel->addOption(
				i18n("Commenting allowance level"),
				i18n("Who except ##0## is allowed to view documents owned by ##0##?", array($viewUser)),
				$values["commentLevel"],
				"commentLevel",
				"select",
				$permissionlevels_array);
			$panel->addOption(
				i18n("Editing allowance level"),
				i18n("Who except ##0## is allowed to view documents owned by ##0##?", array($viewUser)),
				$values["editLevel"],
				"editLevel",
				"select",
				$permissionlevels_array);
			$panel->addOption(
				i18n("Document contents"),
				"",
				$values["content"],
				"content",
				"pagedata");
			
			if( $_POST["saveOptions"] )
			{
				$document["title"] = $values["title"];
				$document["modifydate"] = $document["createdate"] = time();
				$document["blurb"] = $values["blurb"];
				$document["published"] = false;
				$document["backwardrel"] = array();
				$document["forwardrel"] = array();
				$document["owner"] = $viewUser;
				$document["viewLevel"] = $values["viewLevel"];
				$document["commentLevel"] = $values["commentLevel"];
				$document["comments"] = array();
				$document["commentCount"] = 0;
				$document["editLevel"] = $values["editLevel"];
				$document["authors"] = array($viewUser);
				$document["categories"] = array();
				$document["rating"] = $values["rating"];
				
				$theDocument = new dox_document( 0, $document );
				
				$theDocument->saveContent( $values["content"] );
				
				header("Location: " . globalIDtoURL( "user/$viewUser/documents") );
			}
			else
				$renderedContent = $panel->render();
			
			break;
		// List user's documents (owner) - user/username/documents
		default:
			$theDocuments = new dox_alldocs();
			$listed = false;
			$renderedContent = "
			<table class=\"documents-profilelist\">";
			foreach( $theDocuments->documents as $key => $value )
			{
				if( ( is_array($value["authors"]) && in_array( $viewUser, $value["authors"] ) ) || $viewUser == $value["owner"] )
				{
					$listed = true;
					$theDocument = new dox_document( $value["id"] );
					
					if( userAllows( $theDocument->owner, $theDocument->editLevel ) )
					{
						$publishTitle = $theDocument->published ? i18n("Unpublish") : i18n("Publish");
						$editCell = "
						" . drawCommand( i18n("Delete"), i18n("Delete the document"), globalIDtoURL( "user/$viewUser/documents/delete/" . $theDocument->id ) ) . "
						" . drawCommand( i18n("Edit"), i18n("Edit the document"), globalIDtoURL( "user/$viewUser/documents/edit/" . $theDocument->id ) ) . "
						" . drawCommand( $publishTitle, i18n("Toggle the published status of the document"), globalIDtoURL( "user/$viewUser/documents/togglepublish/" . $theDocument->id ) ) . "";
					}
					
					$renderedContent .= "
					<tr class=\"documents-profilelist\">
						<td class=\"documents-profilelist-info\">
						" . $value["title"] . "<small class=\"comment\">" . $value["blurb"] . "</small>
						</td>
						<td class=\"documents-profilelist-commands\">$editCell
						" . drawCommand( i18n("View") . "&raquo", i18n("View the document"), globalIDtoURL( "documents/view/" . $theDocument->id ) ) . "
						</td>
					</tr>";
				}
			}
			$renderedContent .= "
			</table>";
			
			if( !$listed )
				$renderedContent = renderInformationBox( i18n("No documents"), i18n( "##0## has not yet created any documents.", array( getUserInfo( $viewUser, "name" ) ) ) );
	}
	return "<form action=\"" . globalIDtoURL($globalID) . "\" method=\"POST\">" . $renderedContent . "</form>";
}

function modulesetup_documents()
{
	global $pagectl, $setup_folder, $page_folder, $recent_file, $menu_folder, $theModuleSection, $theModuleSubsection, $theModuleAction, $dox_options, $default_language, $permissionlevels_array;
	
	$doxSetup = new TabWidget();
	$doxSetup->tabbar->addTab(i18n("All Documents"), globalIDtoURL("setup/modulesetup/documents"));
	$doxSetup->tabbar->addTab(i18n("Document Ratings"), globalIDtoURL("setup/modulesetup/documents/ratings"));
	$doxSetup->tabbar->addTab(i18n("Document Categories"), globalIDtoURL("setup/modulesetup/documents/categories"));
	$doxSetup->tabbar->addTab(i18n("Options"), globalIDtoURL("setup/modulesetup/documents/options"));
	
	switch( $theModuleSection )
	{
	case "options":
		$doxSetup->tabbar->setCurrent(i18n("Options"));
		if( $theModuleAction == "create_documents_page" )
		{
			// Create the page to save the forum to!
			$edit_page["id"] = $pagectl->getFreePageID($page_folder, $pagectl);
			$edit_page["title"] = $dox_options["title"];
			$edit_page["menu"] = $dox_options["menu_id"];
			$edit_page["language"] = $default_language;
			$edit_page["content"] = "\\\\documents()";
			$edit_page["author"] = getUserInfo(currentUser(), "name");
			$edit_page["email"] = getUserInfo(currentUser(), "email");
			
			unlink_multiple( "$page_folder/{$edit_page['id']}.*" );
			
			if ($pagectl->savePageData($page_folder, $edit_page["id"], $edit_page["language"], $edit_page["author"], $edit_page["email"], $dox_options["menu_id"], $edit_page["title"], $edit_page["content"])) {
				update_getpage_id($page_folder, $setup_folder, $pagectl);
				rotate_recent($edit_page["id"], $edit_page["title"], $edit_page["language"], date("j/n/y"), $recent_file);
				
				$dox_options["page_id"] = $edit_page["id"];
				dox_saveOptions();
				
				header("Location: " . globalIDtoURL("setup/modulesetup/documents/options"));
			} else {
				$error_description = parse_page_data(i18n("The documents page could not be saved!"));
			}
		}
		if( $_POST["saveOptions"] && $_POST["dox"]["page_id"] != 12 && ( $dox_options["page_id"] != $_POST["dox"]["page_id"] || $dox_options["menu_id"] != $_POST["dox"]["menu_id"] || $dox_options["title"] != $_POST["dox"]["title"] ) )
		{
			$thePage = $pagectl->fetchPageData($_POST["page_id"]);
			unlink_multiple( "$page_folder/{$dox_options['page_id']}.*" );
			if ($pagectl->savePageData($page_folder, $_POST["dox"]["page_id"], $thePage["language"], $thePage["author"], $thePage["email"], $_POST["dox"]["menu_id"], $_POST["dox"]["title"], "\\\\documents()"))
			{
				update_getpage_id($page_folder, $setup_folder, $pagectl);
				rotate_recent($dox_options["page_id"], $dox_options["title"], $thePage["language"], date("j/n/y"), $recent_file);
			}
		}
		$panel = new optionsPanel("dox");
		$panel->showHeader = false;
		
		$values["page_id"] = $_POST["dox"]["page_id"] ? $_POST["dox"]["page_id"] : $dox_options["page_id"];
		$values["title"] = $_POST["dox"]["title"] ? stripslashes($_POST["dox"]["title"]) : $dox_options["title"];
		$values["menu_id"] = $_POST["dox"]["menu_id"] ? $_POST["dox"]["menu_id"] : $dox_options["menu_id"];
		$values["rating"] = $_POST["dox"]["rating"] ? $_POST["dox"]["rating"] : $dox_options["rating"];
		$values["viewLevel"] = $_POST["dox"]["viewLevel"] ? $_POST["dox"]["viewLevel"] : $dox_options["viewLevel"];
		$values["commentLevel"] = $_POST["dox"]["commentLevel"] ? $_POST["dox"]["commentLevel"] : $dox_options["commentLevel"];
		$values["editLevel"] = $_POST["dox"]["editLevel"] ? $_POST["dox"]["editLevel"] : $dox_options["editLevel"];
		
		$panel->addHeader( i18n("Documents page"), "thepage" );
		
		$pagelist = $pagectl->fetchPageList($page_folder);
		$temp_pagelist[12] = i18n("Please select a page...");
		foreach( $pagelist as $key => $value )
		{
			$temp_pagelist[$key] = $value["title"][0];
		}
		$panel->addOption(
			i18n("Documents page"),
			i18n("This is the page that will be used to show the documents on. If you want to create a new page to put the information into, ##0##simply click here##1## and this will be done for you.", array("<a href=\"" . globalIDtoURL("setup/modulesetup/documents/options/na/create_documents_page") . "\">", "</a>")),
			$values["page_id"],
			"page_id",
			"select",
			$temp_pagelist
		);
		
		$panel->addOption(
			i18n("Title"),
			i18n("The title of the documents page"),
			$values["title"],
			"title",
			"text"
		);
		
		$menulist = $pagectl->fetchMenuMenuList($menu_folder);
		foreach( $menulist as $key => $value )
		{
			$temp_menulist[$value["id"]] = $value["title"][0];
		}
		$panel->addOption(
			i18n("Documents menu"),
			i18n("This is the menu that will be shown on the document page"),
			$values["menu_id"],
			"menu_id",
			"select",
			$temp_menulist
		);
		
		$panel->addHeader( i18n("Defaults for new users"), "defaults" );
		
		$panel->addOption(
			i18n("Rating"),
			i18n("What rating do new documents fall under by default?"),
			$values["rating"],
			"rating",
			"select",
			$dox_options["ratings"]);
		$panel->addOption(
			i18n("Viewing allowance level"),
			i18n("Who except the user itself is allowed to view documents owned by them?"),
			$values["viewLevel"],
			"viewLevel",
			"select",
			$permissionlevels_array);
		$panel->addOption(
			i18n("Commenting allowance level"),
			i18n("Who except the user itself is allowed to comment on documents owned by them?"),
			$values["commentLevel"],
			"commentLevel",
			"select",
			$permissionlevels_array);
		$panel->addOption(
			i18n("Editing allowance level"),
			i18n("Who except the user itself is allowed to view documents owned by them?"),
			$values["editLevel"],
			"editLevel",
			"select",
			$permissionlevels_array);
		
		if( $_POST["saveOptions"] )
		{
			$dox_options["page_id"] = $_POST["dox"]["page_id"];
			$dox_options["menu_id"] = $_POST["dox"]["menu_id"];
			$dox_options["title"] = $_POST["dox"]["title"];
			
			$dox_options["rating"] = $_POST["dox"]["rating"];
			$dox_options["viewLevel"] = $_POST["dox"]["viewLevel"];
			$dox_options["commentLevel"] = $_POST["dox"]["commentLevel"];
			$dox_options["editLevel"] = $_POST["dox"]["editLevel"];
			
			dox_saveOptions();
		}
		
		$doxSetup->contents = "<form action=\"" . thisPageURL() . "\" method=\"POST\">" . $panel->render() . "</form>";
		
		break;
	case "categories":
		$doxSetup->tabbar->setCurrent(i18n("Document Categories"));
		$doxSetup->contents = renderInformationBox(i18n("Incomplete"), i18n("This is non-completed functionality - sorry for the inconvenience!"));
		// Create top level category
		// Edit title of top level category
		// Delete top level category
		// Create category inside top level category
		// Edit title inside top level category
		// Delete top level category
		break;
	case "ratings":
		$doxSetup->tabbar->setCurrent(i18n("Document Ratings"));
		$doxSetup->addCommand(i18n("New"), i18n("Create new rating"), globalIDtoURL("setup/modulesetup/documents/ratings/new"));
		// Ratings
		switch( $theModuleSubsection )
		{
		case "new":
			$panel = new optionsPanel("");
			$panel->saveTitle = i18n("Save new title");
			$panel->showHeader = false;
			
			$values["title"] = $_POST["title"] ? stripslashes($_POST["title"]) : $dox_options["ratings"][$theModuleAction];
			
			$panel->addOption( i18n("New title"), i18n("Enter the new rating's title here"), $values["title"], "title", "text");
			
			if( $_POST["saveOptions"] && $values["title"] != "")
			{
				$dox_options["ratings"][] = $values["title"];
				dox_saveOptions();
				header("Location: " . globalIDtoURL("setup/modulesetup/documents/ratings"));
			}
			if( $values["title"] == "" )
				$panel->options["title"]["error"] = i18n("You cannot have a rating without a title!");
			
			$renderedContent = renderInformationBox(i18n("Create new rating"), $panel->render(), false);
			break;
		case "edit":
			$panel = new optionsPanel("");
			$panel->saveTitle = i18n("Save new title");
			$panel->showHeader = false;
			
			$values["title"] = $_POST["title"] ? stripslashes($_POST["title"]) : $dox_options["ratings"][$theModuleAction];
			
			$panel->addOption( i18n("New title"), i18n("Enter the rating's new title here"), $values["title"], "title", "text");
			
			if( $_POST["saveOptions"] && $values["title"] != "" )
			{
				$dox_options["ratings"][$theModuleAction] = $values["title"];
				dox_saveOptions();
				header("Location: " . globalIDtoURL("setup/modulesetup/documents/ratings"));
			}
			
			$renderedContent = renderInformationBox(i18n("Edit rating title"), $panel->render(), false);
			break;
		case "delete":
			if( $_REQUEST["confirmDelete"] == "true" )
			{
				unset( $dox_options["ratings"][$theModuleAction] );
				dox_saveOptions();
				header("Location: " . globalIDtoURL("setup/modulesetup/documents/ratings"));
			}
			$renderedContent = renderInformationBox(i18n("Delete this rating?"),
				parse_page_data(i18n("Are you sure you wish to delete the rating with the title ##0##?", array("&quot;" . $dox_options["ratings"][$theModuleAction] . "&quot;"))) .
				"<div class=\"wikicenteralign\">" .
					drawCommand( i18n("Confirm delete"), i18n("Confirm deletion of ##0##", array($dox_options["ratings"][$theModuleAction])), globalIDtoURL( "setup/modulesetup/documents/ratings/delete/$theModuleAction", array("confirmDelete" => "true") ) ) . " " .
					drawCommand( i18n("Abort delete"), i18n("Abort the deletion process"), globalIDtoURL("setup/modulesetup/documents/ratings") ) .
				"</div>",
				false);
			break;
		default:
			$allDocs = new dox_alldocs();
			foreach( $allDocs->documents as $key => $value )
				$ratingCount[$value["rating"]]++;
			
			$renderedContent = "
			<table class=\"setup\">";
			foreach( $dox_options["ratings"] as $key => $value )
			{
				if( $ratingCount[$key] == 0 )
				{
					$deleteThis = drawCommand( i18n("Delete"), i18n("Delete this rating"), globalIDtoURL("setup/modulesetup/documents/ratings/delete/$key") );
					$cantDelete = "<small class=\"comment\">" . i18n("Deletable: 0 documents are rated with this rating") . "</small>";
				}
				else
				{
					$deleteThis = "(" . i18n("Delete") . ")";
					$cantDelete = "<small class=\"comment\">" . i18n("Cannot delete: ##0## documents are rated with this rating", array($ratingCount[$key])) . "</small>";
				}
				
				$renderedContent .= "
				<tr class=\"setup\">
					<td class=\"setup\">
						$value $cantDelete
					</td>
					<td class=\"setup\">
						<div class=\"wikirightalign\">
							" . $deleteThis . "
							" . drawCommand( i18n("Edit"), i18n("Edit this rating"), globalIDtoURL("setup/modulesetup/documents/ratings/edit/$key") ) . "
						</div>
					</td>
				</tr>";
			}
			$renderedContent .= "
			</table>";
			break;
		}
		$doxSetup->contents = "<form action=\"" . thisPageURL() . "\" method=\"POST\">" . $renderedContent . "</form>";
		break;
	default:
		$doxSetup->tabbar->setCurrent(i18n("All Documents"));
		// List all documents
		$allDocs = new dox_alldocs();
		
		$listed = false;
		$renderedContent = "
		<table class=\"documents-profilelist\">";
		foreach( $allDocs->documents as $key => $value )
		{
			$listed = true;
			$theDocument = new dox_document( $value["id"] );
			
			if( userAllows( $theDocument->owner, $theDocument->editLevel ) )
			{
				$publishTitle = $theDocument->published ? i18n("Unpublish") : i18n("Publish");
				$editCell = "
				" . drawCommand( i18n("Delete"), i18n("Delete the document"), globalIDtoURL( "user/$viewUser/documents/delete/" . $theDocument->id ) ) . "
				" . drawCommand( i18n("Edit"), i18n("Edit the document"), globalIDtoURL( "user/$viewUser/documents/edit/" . $theDocument->id ) ) . "
				" . drawCommand( $publishTitle, i18n("Toggle the published status of the document"), globalIDtoURL( "user/$viewUser/documents/togglepublish/" . $theDocument->id ) ) . "";
			}
			
			$renderedContent .= "
			<tr class=\"documents-profilelist\">
				<td class=\"documents-profilelist-info\">
				" . $value["title"] . "<small class=\"comment\">" . $value["blurb"] . "</small>
				</td>
				<td class=\"documents-profilelist-commands\">$editCell
				" . drawCommand( i18n("View") . "&raquo", i18n("View the document"), globalIDtoURL( "documents/view/" . $theDocument->id ) ) . "
				</td>
			</tr>";
		}
		$renderedContent .= "
		</table>";
		
		if( !$listed )
			$renderedContent = renderInformationBox( i18n("No documents"), i18n( "There has not yet been created any documents. If you wish to create one, please visit your profile's Documents panel and click on the New command link, ##0##or simply click here to create a new document##1##", array("<a href=\"" . globalIDtoURL("user/" . currentUser() . "/documents/new") . "\">", "</a>") ) );
		
		$doxSetup->contents = $renderedContent;
	}
	return $doxSetup->renderTabwidget();
}

function dox_saveOptions()
{
	global $dox_options, $dox_optionfile;
	if( file_exists( $dox_optionfile ) )
		unlink( $dox_optionfile );
	
	file_put_contents( $dox_optionfile, array_export( $dox_options, "dox_options" ) );
}

/**
 * Get the next available document ID
 *
 * @return    The document ID
 */
function dox_nextID()
{
	global $dox_options;
	$dox_options["nextID"]++;
	dox_saveOptions();
	return $dox_options["nextID"];
}

function dox_renderListOptions()
{
	global $dox_options, $dox_listingcolumns;
	if( $_POST["saveOptions"] )
	{
		$userShow = array();
		foreach( $_POST["showThis"] as $key => $value )
			$userShow[] = $key;
		saveUserInfo( currentUser(), "dox_showratings", implode( ",", $userShow ) );
		
		$userColumns = array();
		foreach( $_POST["showColumns"] as $key => $value )
			$userColumns[] = $key;
		saveUserInfo( currentUser(), "dox_showcolumns", implode( ",", $userColumns ) );
	}
	else
	{
		$userShow = explode( ",", getUserInfo( currentUser(), "dox_showratings", implode( ",", array_keys($dox_options["ratings"] ) ) ) );
		$userColumns = explode( ",", getUserInfo( currentUser(), "dox_showcolumns", implode( ",", array_values($dox_options["listingColumns"] ) ) ) );
	}
	
	$renderedOptions .= "
	<form action=\"" . thisPageURL() . "\" method=\"POST\">
	<div class=\"documents-listing-options\">
		<div class=\"documents-listing-options-label\">" . i18n("Show documents with the following ratings:") . "</div>
		<div class=\"documents-listing-options-option\">";
	
	foreach( $dox_options["ratings"] as $key => $value )
	{
		$checked = (in_array($key, $userShow)) ? " checked" : "";
		$renderedOptions .= "
		<label><input$checked type=\"checkbox\" name=\"showThis[$key]\" />" . $value . "</label>
		" . drawCommand( i18n("Show"), i18n("Show this rating"), globalIDtoURL("documents/rating/" . $key));
	}
	
	$renderedOptions .= "
		</div>
		<div class=\"documents-listing-options-label\">" . i18n("Show these columns:") . "</div>
		<div class=\"documents-listing-options-option\">";
	
	foreach( $dox_listingcolumns as $key => $value )
	{
		$checked = (in_array($key, $userColumns)) ? " checked" : "";
		$renderedOptions .= "
		<label><input$checked type=\"checkbox\" name=\"showColumns[$key]\" />" . $value . "<label>";
	}
	
	$renderedOptions .= "
		</div>
		<input type=\"submit\" class=\"documents-listing-saveoptions\" name=\"saveOptions\" value=\"" . i18n("Save and apply") . "\" />
	</div>
	</form>";
	
	return $renderedOptions;
}

$modules_globalID[] = "globalID_documents";
function globalID_documents( $splitID )
{
	global $dox_options, $dox_show;
	$pageID = null;
	
	// Is the globalID directly aimed at this module?
	if( $splitID[0] == "documents" )
	{
		$pageID = $dox_options["page_id"];
	}
	// Or is it aimed at the module's profile module?
	else if( $splitID[0] == "user" && $splitID[2] == "documents" )
	{
		$_REQUEST["module"] = "profilemodule_documents";
		$pageID = "13";
	}
	
	return $pageID;
}

/**
 * Return an array of data about the thumbnail for a global ID
 * As the documents don't have thumbnails per se, we simply pass a fitting image directly back
 *
 * @param     splitID        The global ID to check
 *
 * @return    Array containing array
 */
function globalIDThumbnail_documents( $splitID )
{
	global $module_folder;
	$data = array( "uri" => siteURL(true) . "$module_folder/documents/thumb.png", "width" => 64, "height" => 64 );
	return $data;
}
?>
